<?php

namespace App\Services\File\BIM\BlackHole;

use App\Services\File\BIM\Contacts\BIMContact;
use Illuminate\Http\UploadedFile;

class BlackHole implements BIMContact
{
    public function uploadFile(UploadedFile $file, array $params = [])
    {
        $projectId = config("bim.black_hole.project_id");
        $treeInfo = $this->getTreeById($projectId);

        $modelFile = $this->modelFileCreate($projectId, $treeInfo['subNodes'][0]['nodeId'], $file);
        $dataSetId = $modelFile['dataSetId'];

        $uploadFormData = [
            ['name' => 'FileId', 'contents' => $modelFile['fileId']],
            ['name' => 'FileInfo', 'contents' => fopen($file, 'r'), 'filename' => $file->getClientOriginalName()],
            ['name' => 'FileDataId', 'contents' => $modelFile['fileDataId']],
            ['name' => 'Key', 'contents' => $modelFile['extraProperties']['ResumableKey']],
            ['name' => 'BlobIndex', 'contents' => 0],
        ];

        Client::getInstance()->post("/blackHole3D/project/modelFile/append", [
            'multipart' => $uploadFormData,
            'headers' => [
                'fileUploadScheme' => 'Bim',
                'fileUploadMode' => 'ResumableAppend',
            ],
        ]);

        return [
            'bim_data_set_id' => $dataSetId,
            'bim_file_id' => $modelFile['fileId'],
            'bim_convert_status' => 0,
        ];
    }

    public function downloadSourceFile()
    {

    }

    public function viewDataSetModel(array $dataSetIDS)
    {
        $result = Client::getInstance()->post("/blackHole3D/project/dataSet/viewDataSetModel", [
            'dataSetIds' => $dataSetIDS
        ]);

        return $result['data'] ?? [];
    }

    public function findConvertStatus(string $dataSetId)
    {
        $result = Client::getInstance()->post("/blackHole3D/project/dataSet/getConvertStatus", [
            'json' => [
                'dataSetId' => $dataSetId
            ]
        ]);

        return $result['data'] ?? [];
    }

    public function addToConvertQueue(string $dataSetId): array
    {
        $formData = [
            'dataSetId' => $dataSetId,
            'execConvertingCallback' => '',
            'isBuildDbIndex' => true,
            'isResetConverting' => true,
            'paramFilter' => new \stdClass(),
            'fileNames' => [],
            'useDefaultConfig' => false,
        ];

        $result = Client::getInstance()->post("/blackHole3D/project/modelFile/addToConvertQueue", [
            'json' => $formData,
        ]);

        return $result['data'] ?? [];
    }


    protected function modelFileCreate(string $projectId, string $nodeId, UploadedFile $file)
    {
        $formData = [
            'ProjId' => $projectId,
            'DataSetId' => $nodeId,
            'FileName' => uniqid() . $file->getClientOriginalName(),
            'FileSize' => $file->getSize(),
            'FileType' => $file->getExtension() ? $file->getExtension() : pathinfo($file->getClientOriginalName(), PATHINFO_EXTENSION),
            'Scheme' => 'Bim',
            'BlobCount' => 1,
            'BlobSize' => $file->getSize(),
            'UploadWay' => 0
        ];

        $result = Client::getInstance()->post("/blackHole3D/project/modelFile/create", [
            'form_params' => $formData,
            'headers' => [
                'fileUploadScheme' => 'Bim',
                'fileUploadMode' => 'ResumableCreate'
            ]
        ]);

        return $result['data'] ?? [];
    }

    protected function getTreeById(string $projectId)
    {
        $result = Client::getInstance()->post("/blackHole3D/project/modelTree/getTreeById", [
            'json' => [
                'projId' => $projectId
            ]
        ]);

        return $result['data'] ?? [];
    }

    protected function addLeafNodes(array $nodes, string $parentId = "")
    {
        $items = [];
        foreach ($nodes as $node) {
            $items[]['dataSetName'] = $node;
        }

        $result = Client::getInstance()->post("/blackHole3D/project/modelTree/addLevelNodes", [
            'json' => [
                'parentId' => $parentId,
                'dataSets' => $items
            ]
        ]);

        return $result['data'] ?? [];
    }
}
